      subroutine marquardt(iout,nvar,parm,f0)
C
C Minimizing a weighed sum of the squares using Marquardt's method. 
C
C Arguments: iout (integer) - number of the output unit;
C            nvar (integer) - # of variables;
C            parm(nvar) (double precision) - on entry the initial values
C                    of the optimized parameters, on exit their final values; 
C            f0 (double precision) - on exit the final value of the target
C                    function.
C
C You must supply two subroutines: rdif (double precision function)
C that calculates the target function, and HEVAL (subroutine) that 
C calculates the negative of the gradient and the positive-definite
C approximate to the Hessian matrix of the target function: 
C
C      double precision function rdif(nvar,nfun,parm,lder)
C
C where:      nfun is the total # of function calls,
C             lder (logical) is set at .true., if some part of gradient
C             evaluation will be performed during a function call, .false.
C             otherwise.
C
C      subroutine heval(nvar,parm,lsecond) 
C
C where: lsecond (logical) tells whether it is necessary to evaluate the
C        second derivatives (the full Hessian matrix); it helps to speed
C        up the convergence, when the neighborhood of the minimum of the target
C        function has beed achieved. The positive definite approximate to
C        the hessian or full Hessian is stored in the matrix H(maxvar,maxvar)
C        and passed from the subroutine HEVAL via the appropriate common block.
C        It is left to the user to organize this common block and also to define
C        the dimensions.
C
C The algorithm requires solving a system of linear equations. A subroutine
C BANACH that does this by the Banachiewicz decomposition is supplied, but
C the user is free to replace it with another one which he thinks is better.
C In addition to the solution to the linear equation system the subroutine
C should return the logical flag SING which should be .TRUE. when the modified
C matrix H is "numerically" singular.
C
C For the details of the algorithm see: D.W. Marquardt,
C "An algorithm for least-squares estimation of nonlinear parameters."
C J. Soc. Indust. Appl. Math., 11, 431-441, 1963.
C or just any textbook on methods of nonlinear optimization.
C
C A. Liwo, Department of Chemistry, University of Gdansk, 
C ul. Sobieskiego 18, 80-952 Gdansk, Poland
C 
C Send questions and comments to adam@sun1.chem.univ.gda.pl
C
      implicit real*8 (a-h,o-z)
      include 'PARAMET.ECEPP'
      include 'sizesclu.dat'
      include 'PARAMET.FIT' 
      include 'COMMON.MARQ'
      dimension dx(maxvar),diagh(maxvar),parold(maxvar)
      dimension parm(nvar)
      logical sing,lprn
C Set lprn=.true. for debugging (a lot of VERBOSE messages, so don't do this,
C unless ABSOLUTELY necessary).
      lprn=.false.
      nfun=0
      if (maxit.eq.0) maxit=1000
      if (maxmar.eq.0) maxmar=10
      if (rl.eq.0.0D0) rl=100.0D0
      if (vmarq.eq.0.0D0) vmarq=10.0D0
      if (tolx.eq.0.0D0) tolx=1.0D-3
      if (tolf.eq.0.0D0) tolf=1.0D-3
      if (rtolf.eq.0.0D0) rtolf=1.0D-5
      if (tollam.eq.0.0D0) tollam=1.0D0
      call transfer(nvar,parm,parold)
      f0=rdif(nvar,nfun,parm,.true.) 
      write (iout,100) maxit,maxmar,rl,vmarq,tolx,tolf,rtolf,tollam,f0
      if (iout.ne.6)
     &write (*   ,100) maxit,maxmar,rl,vmarq,tolx,tolf,rtolf,tollam,f0
      if (lprn) write (iout,'(/a/5(1pe14.5)/)') 
     & 'Initial parameters:',(parm(i),i=1,nvar)
      do it=1,maxit 
        if (it.gt.1 .and. (ain.le.0.1D0 .or. reldf.le.0.001D0)) then
          call heval(nvar,parm,.true.)
        else
          call heval(nvar,parm,.false.)
        endif
        do i=1,nvar
          do j=i+1,nvar
            h(j,i)=h(i,j)
          enddo
        enddo
        
        if (lprn) then

        write (iout,'(a)') 'The Hessian matrix:'
        call matout(nvar,iout,h)
        write (iout,'(a)') 'Gradient'
        do i=1,nvar
          write (iout,*) 'g(',i,')=',g(i)
        enddo

        endif

        do i=1,nvar
          diagh(i)=h(i,i)
        enddo
        if (rl.gt.rlmin) rl=rl/vmarq
        do imar=1,maxmar
          do i=1,nvar
            h(i,i)=diagh(i)+rl
          enddo
          call transfer(nvar,g,dx)
          call banach(nvar,h,dx,sing) 
          if (sing) write (2,'(a)') 'WARNING! Singularity when solving',
     &  ' the system of normal equations; gradient direction assumed.'
 
          if (lprn) then

          do i=1,nvar
            write (iout,*) 'dx(',i,') =',dx(i)
          enddo

          endif

          ain=0.0D0
          do i=1,nvar
            ain=ain+(dx(i)/dmax1(1.0D0,dabs(parm(i))))**2
          enddo
	  ain=dsqrt(ain/nvar)

          if (lprn) then

          write (iout,'(a)') 'Vector dx:'
          write (iout,'(8f10.5)') (dx(i),i=1,nvar)
          
          endif

          do i=1,nvar
            parm(i)=parold(i)+dx(i)
          enddo

          f1=rdif(nvar,nfun,parm,.true.)
cd        write (iout,'(a,i2,a,1pe8.1,a,1pe14.5)') 
cd   &        'imar=',imar,' rl=',rl,' f1=',f1 
cd        print '(a,i2,a,1pe8.1,a,1pe14.5)', 
cd   &        'imar=',imar,' rl=',rl,' f1=',f1 
          if (f1.lt.f0) then
            df=abs(f1-f0)
            reldf=df
            if (f0.gt.1.0D-13) reldf=reldf/f0
            call transfer(nvar,parm,parold)
            f0=f1
            goto 1
          else if (abs(f1-f0).lt.1.0D-13) then  
            write (iout,500) 
            return
          endif
          rl=rl*vmarq
        enddo
        write (iout,400) 
        call transfer(nvar,parold,parm)
        return
    1   write (iout,200) it,imar,nfun,rl,ain,f0,df,reldf
        if (iout.ne.6)
     &  write (*   ,200) it,imar,nfun,rl,ain,f0,df,reldf
        if ((ain.lt.tolx .or. df.le.tolf .or. reldf.le.rtolf) 
     &                 .and. (rl.le.tollam .or. reldf.le.1.0D-7)) then
          write (iout,300)
          return
        endif
      enddo
  100 format(//' Minimization of the sum of squares',
     & ' using the Marquardt procedure'//
     & '      maxit    maxmar    lambda         V      tolx      tolf',
     & '     rtolf    tollam'
     & /1x,2i10,6(1pe10.1)//
     & ' Initial function value: ',1pe10.4//
     & '    it      imar      nfun    lambda     reldx             f',
     & '        df     reldf'/80(1h-))
  200 format(1x,i5,2i10,2(1pe10.1),1pe14.4,3(1pe10.1))
  300 format(//80(1h-)/' Minimization converged.'/80(1h-)/)
  400 format(//80(1h-)/' Cannot find a lower function value than',
     & ' in a previous step. Minimization terminated.'/80(1h-)/)
  500 format(//80(1h-)/' Target function changes only within the ',
     & ' machine accuracy. Parameters may appear to be interdependent.'
     & /80(1h-)/)
      end  
*************************************************************************** 
      subroutine transfer(n,x1,x2)
      implicit real*8(a-h,o-z)
      dimension x1(n),x2(n)
      do i=1,n
        x2(i)=x1(i)
      enddo
      return
      end      
******************************************************************************
      subroutine matout(n,iout,a)                                       
      implicit real*8 (a-h,o-z)
      include 'PARAMET.ECEPP'
      include 'sizesclu.dat'
      include 'PARAMET.FIT' 
      include 'COMMON.MARQ'
      dimension b(10)
      dimension a(maxvar,maxvar)
      do 1 i=1,n,8                                                      
      nlim=min0(i+7,n)                                                  
      write (iout,1000) (k,k=i,nlim)                                    
      write (iout,1020)                                                 
 1000 format (/9x,8(i7,7x))                                             
 1020 format (/1x,120(1h-)/)                                            
      do 2 j=i,n                                                        
      jlim=min0(j,nlim)                                                 
      jlim1=jlim-i+1                                                    
      do 3 k=i,jlim                                                     
    3 b(k-i+1)=a(j,k)                                   
      write (iout,1010) j,(b(k),k=1,jlim1)                              
    2 continue                                                          
    1 continue                                                          
 1010 format (i5,3x,8(1pe11.4,3x))                                      
      return                                                            
      end
c-----------------------------------------------------------------------------
      subroutine matout1(n,m,iout,a)                                       
      implicit real*8 (a-h,o-z)
      include 'PARAMET.ECEPP'
      include 'sizesclu.dat'
      include 'PARAMET.FIT' 
      include 'COMMON.MARQ'
      dimension b(10)
      dimension a(maxvar,maxvar)
      do 1 i=1,n,8                                                      
      nlim=min0(i+7,n)                                                  
      write (iout,1000) (k,k=i,nlim)                                    
      write (iout,1020)                                                 
 1000 format (/9x,8(i7,7x))                                             
 1020 format (/1x,120(1h-)/)                                            
      do 2 j=1,m                                                        
      write (iout,1010) j,(a(j,k),k=i,nlim)                              
    2 continue                                                          
    1 continue                                                          
 1010 format (i5,3x,8(1pe11.4,3x))                                      
      return                                                            
      end
