********************************************************************************
*                                                                              *
* The code contained in this file was taken directly from the MORASS 2.1       *
* package, which  has been developed by the following people:                  *
*                                                                              *
* R. P. Meadows,  C. B. Post, D. G. Gorenstein, B. A. Luxon                    *
* David G. Gorenstein Research Group                                           *
* Purdue University                                                            *
* West Lafayette, IN USA                                                       *
*                                                                              *
* http://www.nmr.utmb.edu/#mrass                                               *
*                                                                              *
* The code contained in this file belongs exclusively to the above-mentioned   *
* people. If you publish any paper based on the the NMR-fitting option of      *
* ANALYZE please cite:                                                         *
*                                                                              *
* 1. Robert P. Meadows, Carol Beth Post, Bruce A. Luxon,                       *
*    and David G. Gorenstein,                                                  *
*    MORASS 2.1, Purdue University, W. Lafayette (1994).                       *
*                                                                              *
* 2. C.B. Post, R.P. Meadows and D.G. Gorenstein, J. Am. Chem. Soc., 112,      *
*    6796(1990)                                                                *
*                                                                              *
* ANALYZE does not use the I/O routines of MORASS contained in this file.      *
*                                                                              *
*      ------ THIS NOTE MAY NOT BE REMOVED FROM THE SOURCE FILE ------         *
*                                                                              *
********************************************************************************
C ------------------------------------------------------------------------
      SUBROUTINE SIMUL(NWR,ISMMD,GAMF,LGAMF)
C ------------------------------------------------------------------------
C     
C     This subroutine calculates changes in NOESY intensities for 
C     a set of NSP spins through dipolar cross-relaxation.
C     A matrix solution to the Bloch equations with cross-relaxtion is 
C     obtained and the evolution of magnitization for each spin calculated 
C     for a specified set of spins and time period. 
C
C     Cross-relaxation rate constants are defined as:
C                    sigmaAB           
C                   -----> 
C              A             B
C                  <-----          
C                    sigmaBA
C
C     with sigmaAB=sigmaBA
C
C     The following parameters are passed to the routine:
C          TAU		overall rotational correlation time, nanoseconds
C          OM0		field strength, MHz
C          TIME   	maximum mixing time, sec (ISSMD=1)
C                       mixing time, sec         (ISSMD=2)
C          NSP   	number of spins in the system
C          RHO(NSP)	1/T1 = spin-lattice relaxation rate, sec
C                       For methyl protons, use correl time = TAUM ns
C                       NOTE!! methyl selection assumes only geminal
C                       proton pairs are separated by < 1.8 A.
C          CRD(3,NSP)	cartesian coordinates of nuclei, angstroms
C          VOL0(NSP)	equilibrium diagonal peak intensities (i.e. tmix=0.0)
C          ISMMD 	mode of calculation: 
C                        1    time development, write specified vectors
C                 	 2    single mix time, write volume matrix
C
C     On output, the subroutine passes values for the following variables:
C       MAGW(IWR,NSP+1) intensities as a function of mixing time, 
C                       time and values for nsp spins for IWR steps
C       NWR             number of vectors to be written
C       WRMAP(2*MXSP)   mapping array to identify vectors
C
C     Variables used by SIMUL:
C       GAMMAS(NSP,NSP)  relaxation matrix. Diagonal elements = rho,
C                       off-diagonal i,j elements = sigma(i,j).  GAMMA
C                       is symmetrical.
C       LMBDA(NSP)	eigenvalues of GAMMA
C       REVEC(NSP,NSP)  eigenvectors of GAMMA
C
C  AVERAGEING SIGMA  NOTE:
C
C  WILL USE THE SIGMA AVERAGE "SAVG" STRATEGY FOR OTHER CONSTRAINTS (eg ORTHO
C  RING PROTONS) AS FOR THE METHYLS, HOWEVER, THIS WILL NOT BE ABLE TO BE
C  DONE TRANSPARENTLY AS WITH THE METHYLS BUT WILL REQUIRE USER INPUT
C  IN ORDER TO BE GENERAL ENOUGH TO MAKE IT WORTHWHILE WRITING THE CODE.
C
C                                                   B.A.LUXON  9/20/93
C ------------------------------------------------------------------------
C
      INCLUDE 'molecule.inc'
      INCLUDE 'submol.inc'
C 
      REAL           TINC,OM0N
      REAL           EVINV(MXSP,MXSP),REVEC(MXSP,MXSP),LMBDA(MXSP),
     &     LPR(MXSP,MXSP),LMBT(MXSP)
      REAL           SPECD,JZERO,JONE,JTWO,SUMJ,RIJ6(MXSP,MXSP)
      REAL           JMZER,JMONE,JMTWO,RIJA(9),RIJI6,SUMJM
      INTEGER        IDGT,NWR,NUMINC,CHK,OUT,COLID,COLJD
      CHARACTER*(*)  GAMF
      LOGICAL        METHYL,METHYLJ,METHYLI,SAVGI,SAVGJ
      CHARACTER*1    IJB
C
      PARAMETER(CHK=22,OUT=20)
      PARAMETER(SIXTH=1./6.)
C
      PI=ACOS(-1.)
      CNST=56.9509
      GMDST=1.8**6
      SAVGI = .FALSE.
      SAVGJ = .FALSE.
C
      NUMINC=MXWR-1
      TINC=TIME/FLOAT(NUMINC)
C
C ------------------------------------------------------------------------
C     Calculate sigma and rho for explicit dipole-dipole interactions
C       from TAU, OMEGA and coordinates.
C
C     The constant CNST if for proton-proton and uses the following values:
C         gyromag. ratio = 26,753 (proton)
C         planck's cnst  = 1.0544E-27
C         conversion to angstrom**6 and nanosecond = E+39 (i.e. 48-9)
C ------------------------------------------------------------------------
C
      OM0N = OM0*1.E-3*2.*PI
      OM2 = 2.0*OM0N
      JZERO = SPECD(0.0,TAU)
      JONE = SPECD(OM0N,TAU)
      JTWO = SPECD(OM2,TAU)
      JMZER = SPECD(0.0,TAUM)
      JMONE = SPECD(OM0N,TAUM)
      JMTWO = SPECD(OM2,TAUM)
C
C ------------------------------------------------------------------------
C  Calculate the sigma values for all proton pairs. For now, fill
C  both triagonals of gamma, this may be deleted once a reliable
C  routine is called with the convention for symmetrical matrices.
C
C  NOTE: The methyl algorithm assumes Me protons are
C         sequential in coordinate file.
C
C  NOTE: ADDED OTHER PROTON EQUIVALENCES FOR MOTIONAL AVERAGING
C        CALLING THE 'SIGAVG' ROUTINE            B.A.LUXON  9/21/93
C ------------------------------------------------------------------------
C
      SUMJ = 6.*JTWO-JZERO
      SUMJM = 6.*JMTWO-JMZER
C
      DO 120 I=1,NSP
         CALL METH(I,'CASP',METHYLI,IS,IL,INME)
cd       write (20,*) 'i=',i,' methyli=',methyli,' is=',is,' il=',il
C
C IF NOT A METHYL SEE IF WE HAVE OTHER PROTONS TO AVERAGE
C
         IF (.NOT. METHYLI .AND. SAVGSW) THEN
           IJB = 'I'
           CALL SAVG(IJB,I,SAVGI,IS,IL,COLID)
         ENDIF
C
C ----
C Loop over second atom for numbers > than first atom.
C ----
C
         JSTART=I+1
C
         DO 100 J=JSTART,NSP
            CALL METH(J,'CASP',METHYLJ,JS,JL,JNME)
C
            IF (.NOT. METHYLJ .AND. SAVGSW) THEN
              IJB = 'J'
              CALL SAVG(IJB,J,SAVGJ,JS,JL,COLJD)
            ENDIF 
C 
C 
C ---- 
C Get average 1/r**6  (for non methyls, is=il or js=jl)
C ----
C
            KK=0
C
            IF(SAVGI .OR. SAVGJ) THEN
C
C TOP OF IF LOOP IF DOING SIGMA AVERAGING ON NON-METHYL EQUIVALENT PROTONS
C
              IF(SAVGI.AND..NOT.SAVGJ) THEN
C
            NN=0
               DO 301 JBS = JS,JL
                NN = NN + 1
                JCRD(1,NN) = CRD(1,JBS)
                JCRD(2,NN) = CRD(2,JBS)
                JCRD(3,NN) = CRD(3,JBS)
 301           CONTINUE
               JS = 1
               JL = NN
C
              ELSEIF (SAVGJ.AND..NOT.SAVGI) THEN
C
            NN=0
               DO 302 IBS = IS,IL
                NN = NN + 1
                ICRD(1,NN) = CRD(1,IBS)
                ICRD(2,NN) = CRD(2,IBS)
                ICRD(3,NN) = CRD(3,IBS)
 302           CONTINUE
               IS = 1
               IL = NN
C
              ENDIF
C
              DO 304 II=IS,IL
                DO 303 JJ=JS,JL
C                IF(JJ .NE. II) THEN
                   KK=KK+1
                   DELX = ICRD(1,II)-JCRD(1,JJ)
                   DELY = ICRD(2,II)-JCRD(2,JJ)
                   DELZ = ICRD(3,II)-JCRD(3,JJ)
                   RIJA(KK) = DELX**2+DELY**2+DELZ**2
C                ENDIF
 303            CONTINUE
 304           CONTINUE
C
            ELSE
C
C BOTTOM OF IF LOOP IF DOING NORMAL SIGMA OR METHYL AVERAGING
C
              DO 306 II=IS,IL
                DO 305 JJ=JS,JL
                 KK=KK+1
                 DELX = CRD(1,II)-CRD(1,JJ)
                 DELY = CRD(2,II)-CRD(2,JJ)
                 DELZ = CRD(3,II)-CRD(3,JJ)
                 RIJA(KK) = DELX**2+DELY**2+DELZ**2
 305            CONTINUE
 306           CONTINUE
            ENDIF
C
C ----
C Account for intramethyl averaging which includes 3 zero-distance pairs
C & Account for equiv proton averaging which includes n zero-distance pairs
C ----
C
            RIJI6=0.
C
C INTRAMETHYL AVERAGING
C
            IF (IS.EQ.JS .AND. METHYLI) THEN
              DO 307 II=2,4
                RIJI6=RIJI6+1.0/RIJA(II)**3
 307          CONTINUE
              DO 308 II=6,8
                RIJI6=RIJI6+1.0/RIJA(II)**3
 308          CONTINUE
              KK=6
C
C INTRA-EQUIVALENT PROTON AVERAGING
C
            ELSEIF(SAVGI .AND. SAVGJ .AND. (COLID .EQ. COLJD)) THEN
              DO 309 II = 1,KK
                  IR = (II+(IL-1))/IL
                  IOUT = IR + (IL*(IR - 1))
                  IF(II.NE.IOUT) RIJI6=RIJI6+1.0/RIJA(II)**3
 309          CONTINUE
              KK = (IL * IL) - IL
C
C NORMAL PROTON AVERAGING OK FOR ALL OTHER CASES
C
            ELSE
              DO 310 II=1,KK
                RIJI6=RIJI6+1.0/RIJA(II)**3
 310          CONTINUE
            ENDIF
C
            RIJI6=RIJI6/FLOAT(KK)
            RIJ6(I,J) = 1.0/RIJI6
            RIJ6(J,I) = 1.0/RIJI6
C------
C save RIJ from simul for writing out and for analysis
C------
            RIJ=1.0/RIJI6**SIXTH
            SRIJ(I,J)=RIJ
            SRIJ(J,I)=RIJ
C
            IF (RIJ6(I,J).LT.GMDST .AND. METHYLI) THEN
              SIGMA = SSQ(I,J)*CNST*SUMJM/RIJ6(I,J)
            ELSE
              SIGMA = SSQ(I,J)*CNST*SUMJ/RIJ6(I,J)
            ENDIF
C
            IF (SIGMA .GT. -1.E-4) SIGMA=0.0
C
            GAMMAS(J,I) = SIGMA
            GAMMAS(I,J) = SIGMA
 100     CONTINUE
 120  CONTINUE
C
C ------------------------------------------------------------------------
C  Rho values for explicit DD interactions (RO). External interactions
C  will be treated separately (6/10/89 RHO(i)).
C  Rho values for methyl-methyl interactions use tau=TAUM for internal
C  rotation of methyl rotors (Jan 90, cbp)
C ------------------------------------------------------------------------
C
      SUMJ=JZERO + 3.*JONE + 6.*JTWO
      SUMJM=JMZER + 3.*JMONE + 6.*JMTWO
C
      DO 160 I=1,NSP
         RO=0.0
         CALL METH(I,'CASP',METHYL,NUM1,NUM2,INLOC)
C
         DO 140 J=1,NSP
            IF (I.NE.J) THEN
              IF (RIJ6(I,J).LT.GMDST .AND. METHYL) THEN
                 RO=RO+(SSQ(I,J)*CNST*SUMJM/RIJ6(I,J))
              ELSE
                 RO=RO+(SSQ(I,J)*CNST*SUMJ/RIJ6(I,J))
              ENDIF
            ENDIF
 140     CONTINUE
C
         IF (RO.GT.RHO(I)) THEN
           GAMMAS(I,I)=RO
         ELSE
           GAMMAS(I,I)=RHO(I)
         ENDIF
 160  CONTINUE
C
C ------------------------------------------------------------------------
C  AVERAGE THE DIAGONAL (RHO) VALUES FOR EQUIVALENT PROTONS
C ------------------------------------------------------------------------
C
      IF (SAVGSW) THEN
      DO 180 I=1,SVCNT
        ROSUM = 0.0
        J = SVLIST(1,I)
C
C CALCULATE THE AVERAGE GAMMAS VALUE FOR EQUIVALENT PROTONS
C
        DO 170 K = 1,J
          II = K + 1
          JJ = SVLIST(II,I)
          ROSUM = ROSUM + GAMMAS(JJ,JJ)
 170    CONTINUE
        ROSUM = ROSUM/FLOAT(J)
C
C REPLACE GAMMAS WITH THE AVERAGED VALUE
C
        DO 175 K = 1,J
          II = K + 1
          JJ = SVLIST(II,I)
          GAMMAS(JJ,JJ) = ROSUM 
 175    CONTINUE
 180  CONTINUE
      ENDIF
C
C ------------------------------------------------------------------------
C  Now solve the matrix equations. First we'll get both the 
C  eigenvalues and eigenvectors of GAMMA from the mathematics routine.
C ------------------------------------------------------------------------
C
      CALL DIAG(GAMMAS,NSP,REVEC,LMBDA,CHK,IER)
C
C ------
C Invert REVEC using IMSL subroutine.
C ------
C
      DO 480 I=1,NSP
         DO 460 J=1,NSP
            MTMP(J,I)=REVEC(J,I)
 460     CONTINUE
 480  CONTINUE
C
      CALL LINV1F(MTMP,NSP,MXSP,EVINV,IDGT,WK,IER)
C
C -----
C Check inversion of eigenvector matrix (if CHECK).
C -----
C
      IF(CHECK)THEN
         OPEN(UNIT=CHK,FILE='simul.chk',STATUS='NEW')
         WRITE(CHK,'('' '')')
         WRITE(CHK,'('' Eigenvector Inverse: '')')
         DO 500 I=1,NSP
            WRITE(CHK,240) (EVINV(I,J),J=1,NSP)
 500     CONTINUE
 240     FORMAT(8(F10.5))
C
         CALL MATMUL(REVEC,MXSP,EVINV,MXSP,NSP,NSP,NSP,LPR,MXSP)
C
         WRITE(CHK,'('' '')')
         WRITE(CHK,'('' EVECT*EVECT-1: '')')
         DO 520 I=1,NSP
            WRITE(CHK,240) (LPR(I,J),J=1,NSP)
 520     CONTINUE
      ENDIF
      CLOSE(UNIT=CHK)
C
C ------------------------------------------------------------------------
C     Choose calculation mode:
C         ISMMD = 1,   do time dependence
C                 2,   do single time point and write out VOL2D
C          sum=EVECT*exp(lamda*TT)*EVINV
C ------------------------------------------------------------------------
C
      IF(ISMMD.EQ.1)THEN
         TT=0.0
         NWR2=NWR*2
C
C -----
C For each increment, calculate the specified volume as a function of
C mixing time.
C -----
C
         DO 800 IWR=1,NUMINC+1
            DO 560 I=1,NSP
              LMBT(I)=EXP(-LMBDA(I)*TT)
 560        CONTINUE
C
            MAGW(IWR,1)=TT
            DO 720 I=2,NWR2,2
              KWR1=WRMAP(I-1)
              KWR2=WRMAP(I)
              SUM=0.0
C
              DO 570 K=1,NSP 
                   SUM=SUM+REVEC(KWR1,K)*LMBT(K)*EVINV(K,KWR2)
 570          CONTINUE
C
              VOL2DS(KWR1,KWR2)=SUM*VOL0(KWR2)
              KK=I/2 + 1
              MAGW(IWR,KK)=VOL2DS(KWR1,KWR2)
 720        CONTINUE
            TT=TT+TINC
 800     CONTINUE
      ELSEIF (ISMMD.EQ.2) THEN
C
C ----
C  Get a single mixing time volume matrix.
C ----
C
         TT=TIME      
         DO 820 I=1,NSP
            LMBT(I)=EXP(-LMBDA(I)*TT)
 820     CONTINUE
C
         DO 900 J=1,NSP
            DO 880 I=1,J
               SUM=0.0
               DO 870 K=1,NSP 
                  SUM=SUM+REVEC(I,K)*LMBT(K)*EVINV(K,J)
 870           CONTINUE
               VOL2DS(I,J)=SUM*VOL0(J)
 880        CONTINUE
 900      CONTINUE
C
          DO 940 J=1,NSP
            DO 920 I=1,J
              VOL2DS(J,I)=VOL2DS(I,J)
 920        CONTINUE
 940     CONTINUE
      ENDIF 
C
      RETURN
      END
C
C ------------------------------------------------------------------------
      FUNCTION SPECD(OMEGA,TAU)
C ------------------------------------------------------------------------
C     SPECTRAL DENSITY FOR EXPONENTIAL CORRELATION FUNCTION.
C
      REAL OMEGA,TAU,SPECD
      DENOM=1+(OMEGA*TAU)**2
      SPECD=TAU/DENOM
      RETURN
      END
C
C ------------------------------------------------------------------------
      SUBROUTINE CHKEIG(CIDENT,OPER,M,VECT,VAL,NV,PRG,CHK)
C ------------------------------------------------------------------------
C     Check eigenvalue-eigenvector relationship with operator.
C     Assumed order of multiplication is OPER*VECT=VECT*VAL
C         OPER,VECT are of dimension MxM, full storage mode.
C         NV eigenvalues so that VAL is vector of length NV.
C     The matrix PRG is used for answer and is in the calling list
C         since it is adjustable array
C ------------------------------------------------------------------------
C
      DIMENSION OPER(M,M),VECT(M,M),VAL(NV)
      DIMENSION PRG(M,M)
      INTEGER CHK
      CHARACTER CIDENT*8
C
      WRITE(CHK,'('' '')')
      WRITE(CHK,'('' EIGENVALUES CHECKED FROM '',A8)')CIDENT
      DO 340 J=1,NV
            WRITE(CHK,'(2(F12.4,2X))') VAL(J)
 340     CONTINUE
C
      WRITE(CHK,'('' '')')
      WRITE(CHK,'('' EIGENVECTORS CHECKED FROM '',A8)')CIDENT
      DO 360 I=1,NV
         WRITE(CHK,'(4(F9.4,F9.4,1X))') (VECT(I,J),J=1,NV)
 360  CONTINUE
C
C -----
C Operator times eigenvector.
C -----
C
      CALL MATMUL(OPER,M,VECT,M,NV,NV,NV,PRG,M)
C
      WRITE(CHK,'('' '')')
      WRITE(CHK,'('' GAMMA*EIGENVECTORS: CHECKED FROM '',A8)') CIDENT
      DO 600 I=1,NV
         WRITE(CHK,'(8(F9.4,1X))') (PRG(I,J),J=1,NV)
 600  CONTINUE
C
C -----
C Eigenvector times eigenvalue.
C -----
C
      DO 500 J=1,NV
         DO 490 I=1,NV
            PRG(I,J)=VECT(I,J)*VAL(J)
 490     CONTINUE
 500  CONTINUE
C
      WRITE(CHK,'('' '')')
      WRITE(CHK,'('' EIGENVECTOR*EIGENVALUE: CHECK FROM '',a8)')CIDENT
      DO 640 I=1,NV
         WRITE(CHK,'(8(F9.4,1X))') (PRG(I,J),J=1,NV)
 640  CONTINUE
      RETURN
      END
C
C
C ------------------------------------------------------------------------
      SUBROUTINE MATMUL(A1,IA1,A2,IA2,N,M,L,AOUT,IAOUT)
C ------------------------------------------------------------------------
C
C     Multiply matrix A1 by A2:    A1(M,N) x A2(N,L) = AOUT(M,L)
C
C      Input:    A1 row dimension in full storage mode is IA1
C                A2                  "                    IA2
C      Output:   AOUT                "                    IAOUT
C
C     cbpost 7/89
C ------------------------------------------------------------------------
C
      DIMENSION A1(IA1,N),A2(IA2,L),AOUT(IAOUT,L)
C
      DO 200 I3=1,L
        DO 210 I1=1,M
          SUM=0.0
          DO 220 I2=1,N
            SUM=SUM+A1(I1,I2)*A2(I2,I3)
  220     CONTINUE
          AOUT(I1,I3)=SUM
  210   CONTINUE
  200 CONTINUE
C
      RETURN
      END
